

#include <glib-2.0/glib.h>
#include <stdio.h>


static void 
g_date_update_dmy (GDate *d)
{
  GDateYear y;
  GDateMonth m;
  GDateDay day;
  
  guint32 A, B, C, D, E, M;
  
  g_return_if_fail (d != NULL);
  g_return_if_fail (d->julian);
  g_return_if_fail (!d->dmy);
  g_return_if_fail (g_date_valid_julian (d->julian_days));
  
  /* Formula taken from the Calendar FAQ; the formula was for the
   *  Julian Period which starts on 1 January 4713 BC, so we add
   *  1,721,425 to the number of days before doing the formula.
   *
   * I'm sure this can be simplified for our 1 January 1 AD period
   * start, but I can't figure out how to unpack the formula.  
   */
  
  A = d->julian_days + 1721425 + 32045;
  B = ( 4 *(A + 36524) )/ 146097 - 1;
  C = A - (146097 * B)/4;
  D = ( 4 * (C + 365) ) / 1461 - 1;
  E = C - ((1461*D) / 4);
  M = (5 * (E - 1) + 2)/153;
  
  m = M + 3 - (12*(M/10));
  day = E - (153*M + 2)/5;
  y = 100 * B + D - 4800 + (M/10);
  
#ifdef G_ENABLE_DEBUG
  if (!g_date_valid_dmy (day, m, y)) 
    {
      g_warning ("\nOOPS julian: %u  computed dmy: %u %u %u\n", 
		 d->julian_days, day, m, y);
    }
#endif
  
  d->month = m;
  d->day   = day;
  d->year  = y;
  
  d->dmy = TRUE;
}



guint        
g_date_get_monday_week_of_year2 (GDate *d)
{
  GDateWeekday wd;
  guint day;
  GDate first;
  guint ret;
  
  g_return_val_if_fail (d != NULL, 0);
  g_return_val_if_fail (g_date_valid (d), 0);
  
  if (!d->dmy) 
    {
      g_date_update_dmy (d);
    }
  g_return_val_if_fail (d->dmy, 0);  
  
  g_date_clear (&first, 1);
  
  g_date_set_dmy (&first, 1, 1, d->year);
  
  wd = g_date_get_weekday (&first) - 1; /* make Monday day 0 */
  day = g_date_get_day_of_year (d) - 1;
  
ret =  ((day + wd)/7U + (wd < 4 ? 1 : 0));
 if (ret == 0)
 {
  /*  year = d->year - 1; */
   g_date_clear (&first, 1);
  g_date_set_dmy (&first, 31, 12, d->year - 1);
  return g_date_get_monday_week_of_year2 (&first);
 }
 wd = g_date_get_weekday (d);
 if ((ret == 53) && (wd < 4))
 {
 g_date_clear (&first, 1);
  g_date_set_dmy (&first, 1, 1, d->year + 1);
  return g_date_get_monday_week_of_year2 (&first);
} 
 return ret;
}



guint        
g_date_get_sunday_week_of_year2 (GDate *d)
{
  GDateWeekday wd;
  guint day;
  GDate first;
  guint ret;
  
  g_return_val_if_fail (d != NULL, 0);
  g_return_val_if_fail (g_date_valid (d), 0);
  
  if (!d->dmy) 
  {
    g_date_update_dmy (d);
  }
  g_return_val_if_fail (d->dmy, 0);  
  
  g_date_clear (&first, 1);
  
  g_date_set_dmy (&first, 1, 1, d->year);
  
  wd = g_date_get_weekday (&first);
  if (wd == 7) wd = 0; /* make Sunday day 0 */
  day = g_date_get_day_of_year (d) - 1;
  
  ret = ((day + wd)/7U + (wd < 5 ? 1 : 0));
  if (ret == 0)
  {
    /*  year = d->year - 1; */
    g_date_clear (&first, 1);
    g_date_set_dmy (&first, 31, 12, d->year - 1);
    return g_date_get_sunday_week_of_year2 (&first);
  }
  wd = g_date_get_weekday (d);
  if ((ret == 53) && ((wd < 4) || (wd==7)) )
  {
    g_date_clear (&first, 1);
    g_date_set_dmy (&first, 1, 1, d->year + 1);
    return g_date_get_sunday_week_of_year2 (&first);
  } 
  return ret;
}



gint main(int   argc,
	  char *argv[] )
{
  GDate date;
  guint  year, month, day; 
  gint i;
  gchar buffer[100];
  

  while (year != 1234)
  {
   printf("YEAR   : "); scanf("%d", &year);
    day = 22;
    month=12;

    printf("\t\t\t    Monday   Sunday   Monday2   Sunday2\n");
    
    g_date_clear(&date,1);
    g_date_set_dmy(&date,day, month, year);

    for (i=0;i<20;i++)
    {
      g_date_add_days(&date, 1);
      g_date_strftime(buffer, sizeof(buffer), "%d/%m/%Y  %a  :  ", &date);
      printf("DATE %s   ", buffer);

      printf("   %2d   -  ", g_date_get_monday_week_of_year(&date));
      printf("  %2d  |  ", g_date_get_sunday_week_of_year(&date));

      printf("  %2d   -  ", g_date_get_monday_week_of_year2(&date));
      printf("%2d\n", g_date_get_sunday_week_of_year2(&date)); 
    }
  }
  return(0);
}

